
class Hermes::Api::Dmo::HousingRequestSharesController < Hermes::Api::Dmo::ApplicationController
  VALID_OPERATORS = %w[$in $nin $ne $gt $gte $lt $lte $eq $regex $all]

  ACCEPTABLE_PARAMS = %w[
    housing_request_id
    lessor_id
    code
    expire_at
    created_at
    updated_at
  ]

  MUTATE_PARAMS = %i[
    housing_request_id
    lessor_id
  ]

  def mutate_params
    MUTATE_PARAMS
  end 

  def find_by_housing_request_id
    return render_json_error(:unprocessable_entity, :params_housing_request_id_not_found) unless params[:housing_request_id].present?
    
    @housing_request_share = Hermes::HousingRequestShare.order_by(expire_at: :desc).find_by(housing_request_id: params[:housing_request_id])
    return render_json_error(:not_found, :instance_not_found) unless @housing_request_share
    return render_json_error(:unauthorized, :no_access) unless can_access?

    render jsonapi: @housing_request_share
  end

  def can_access?
    if current_user.auth_roles&.include?('superadministrateur') || current_user.roles&.include?('Administrator')
      return true
    end

    if current_user.auth_roles&.include?('bailleur') && @housing_request_share.lessor_id.to_s == current_user.lessor_id.to_s
      return true
    end

    false
  end
end 
