class Hermes::Api::Dmo::NotificationCenter::SubscriptionsController < Hermes::Api::Dmo::ApplicationController
  include ::NotificationsApiCommonController
  before_action :set_subscription, only: %i[show subscribe unsubscribe update_keys]
  before_action :set_params, only: %i[subscribe unsubscribe]

  def index
    paginate(
      ::NotificationCenter::Subscription.where(subscriber_id: @targetable.id),
      each_serializer: serializer,
    )
  end

  def show
    render(jsonapi: @subscription, serializer: serializer)
  end

  def subscribe
    update_subscription('subscribe')
  end

  def unsubscribe
    update_subscription('unsubscribe')
  end

  def update_keys
    return render_json_error(:not_found, :object_not_found)  unless @subscription
    return render_json_error(:unprocessable_entity, :missing_keys) unless keys = params.symbolize_keys[:keys]

    wrong_subscribed_params = keys.any? { |p| !p.symbolize_keys.key?(:subscribed) || ![true, false].include?(p.symbolize_keys[:subscribed]) }
    return render_json_error(:unprocessable_entity, :wrong_subscribed_params) if wrong_subscribed_params

    wrong_provider_params = keys.any? { |p| p.symbolize_keys.key?(:provider) && !['email', 'slack', 'sms'].include?(p.symbolize_keys[:provider]) }
    return render_json_error(:unprocessable_entity, :wrong_provider_params) if wrong_provider_params

    targetable_keys = ::NotificationCenter::Subscription.keys(@targetable)
    wrong_key_params = keys.any? { |p| !p.symbolize_keys.key?(:key) || !targetable_keys.include?(p.symbolize_keys[:key]) }
    return render_json_error(:unprocessable_entity, :wrong_key_params) if wrong_key_params

    keys.each do |p|
      p = p.symbolize_keys
      action_type = p[:subscribed] ? 'subscribe' : 'unsubscribe'
      provider = p[:provider]&.to_sym || :email
      @subscription.send("#{action_type}_to_#{provider}", p[:key])
    end

    render(jsonapi: @subscription.reload, serializer: serializer)
  end 

  protected

  def serializer
    ::Hermes::NotificationCenter::SubscriptionSerializer
  end

  def update_subscription(action_type)
    case @provider
    when :email
      @targetable.subscription.send("#{action_type}_to_email", @key)
    when :slack
      @targetable.subscription.send("#{action_type}_to_slack", @key)
    else
      return render_json_error(:not_found, :object_not_found)
    end

    @subscription.reload
    render(jsonapi: @subscription, serializer: serializer)
  end
end