# frozen_string_literal: true

# Return Hermes::HousingRequest attributes from parsed xml

module Hermes
  module BoClient
    module Mappings
      class HousingRequestSupplementMap   
        include Hermes::BoClient::MappingHelper
             
        def initialize(xml)
          @xml = xml
        end

        def build_housing_request
          attributes = {
            cotations: {},
            supplement_status: nil
          }
          complementDemande = @xml['getComplementDemandeLogementResponse']
          localisationsSouhaitees = transform_in_array(complementDemande.dig('localisationsSouhaitees', 'localisationSouhaitee'))

          localisationsSouhaitees&.each do |loc|
            next unless loc
            attributes[:cotations][loc['commune'].to_s] = loc['nbreDePoints'].to_i if loc['nbreDePoints'].present?
          end

          attributes[:supplement_status] = complementDemande['etatDemande']['libelleEtatDemande']
          attributes[:supplement_can_apply] = complementDemande['etatDemande']['candidatAutorise']

          attributes
        end
      end
    end
  end
end
