require 'rails_helper'

RSpec.describe ::User, type: :model do
  describe 'Lessor association' do
    let!(:entity1) { create(:entity) }
    let!(:user_lessor) { create(:user, first_name: 'user_lessor', group_ids:[entity1.manager_group_id]) }
    let!(:user_lessor_associated) { create(:user) }
    let!(:user_not_associated) { create(:user) }
    let!(:lessor) { create(:lessor, user_id: user_lessor.id) }
    
    it 'can associate lessor and user' do
      user_lessor_associated.lessor = lessor
      user_lessor_associated.save
      user_lessor_associated.reload
      user_lessor.reload
      expect(user_lessor.lessor).to eq(lessor)
      expect(user_lessor_associated.lessor).to eq(lessor)
    end

    it 'should return true or false dependant of lessor association' do
      user_lessor.reload
      user_not_associated.reload
      expect(user_lessor.has_lessor?).to be(true)
      expect(user_not_associated.has_lessor?).to be(false)
    end

    it 'create user_right' do
      expect(Hermes::UserRight.all.count).to be(1)
      user_lessor.group_ids = []
      user_lessor.save
      expect(Hermes::UserRight.all.count).to be(0)
    end
  end

  describe 'check_roles' do 
    let!(:user) { create(:user, auth_roles: ['salarie', 'entreprise']) }

    it 'remove entreprise' do 
      expect(user.auth_roles).to eq(['salarie', 'entreprise'])
    end

    it 'remove entreprise' do 
      user.update(auth_roles: ['salarie', 'entreprise', 'cre'])
      expect(user.auth_roles).to eq(['salarie', 'cre'])
    end

    it 'remove bailleur' do 
      user.update(auth_roles: ['salarie', 'bailleur', 'rdsl'])
      expect(user.auth_roles).to eq(['salarie', 'rdsl'])
    end

  end

  describe 'test group_ids' do 
    let!(:user) { create(:user)}
    let!(:entity) { create(:entity)}
    (1..100).each do |i|
      let!(:"entity_#{i}") { create(:entity)}
      let!(:"user_right_#{i}") { create(:user_right, user: user, entity: eval("entity_#{i}"), type: 'entity', right: 'administrator' )}
    end


    it 'works' do
      user.update(group_ids: user.group_ids + [entity.manager_group_id] - [entity_1.manager_group_id])
      expect(user.reload.group_ids.include?(entity.manager_group_id)).to eq(true)
      expect(user.reload.group_ids.include?(entity_1.manager_group_id)).to eq(false)
    end
  end 


  describe 'default provider' do 
    let(:user) { create(:user)}

    it 'works' do 
      expect(HERMES_CONFIG['identity_provider_key']).to eq('als_hermes_salarie')
      expect(user.reload.provider).to eq('als_hermes_salarie')
    end 
  end 
end
